# Copyright 2019-2020 Turner Broadcasting Inc. / WarnerMedia
# Copyright 2021 Chris Farris <chrisf@primeharbor.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import boto3
from botocore.exceptions import ClientError
from boto3.dynamodb.conditions import Key
import json
import os
import logging

from .vpc import *
from .config import *

import logging
logger = logging.getLogger('antiope.AWSAccount')


class AWSAccount(object):
    """Class to represent an AWS Account """
    def __init__(self, account_id, config=None):
        """ Create a new object representing the AWS account specified by account_id """
        # Execute any parent class init()
        super(AWSAccount, self).__init__()

        self.account_id = account_id

        if config is None:
            # Assume we're able to get what we need from EnvVars
            self.config = AntiopeConfig()
        else:
            self.config = config

        # # Save these as attributes
        self.account_table = self.config.account_table
        self.vpc_table     = self.config.vpc_table
        self.default_session_name = self.config.role_session_name

        if self.config.role_name is None:
            self.cross_account_role_arn = None
        else:
            self.cross_account_role_arn = "arn:aws:iam::{}:role/{}".format(self.account_id, self.config.role_name)

        response = self.account_table.query(
            KeyConditionExpression=Key('account_id').eq(self.account_id),
            Select='ALL_ATTRIBUTES'
        )
        try:
            self.db_record = response['Items'][0]
            # Convert the response into instance attributes
            self.__dict__.update(self.db_record)
            # self.account_name = str(self.account_name.encode('ascii', 'ignore'))
        except IndexError as e:
            raise AntiopeAccountLookupError(f"ID {account_id} not found")
        except Exception as e:
            logger.error("Got Other error: {}".format(e))

    def __str__(self):
        """when converted to a string, become the account_id"""
        return(self.account_id)

    def __repr__(self):
        """Create a useful string for this class if referenced"""
        return("<AWSAccount {} >".format(self.account_id))

    #
    # Cross Account Role Assumption Methods
    #
    def get_creds(self, session_name=None):
        """
        Request temporary credentials for the account. Returns a dict in the form of
        {
            creds['AccessKeyId'],
            creds['SecretAccessKey'],
            creds['SessionToken']
        }
        Which can be passed to a new boto3 client or resource.
        Takes an optional session_name which can be used by CloudTrail and IAM
        Raises AntiopeAssumeRoleError() if the role is not found or cannot be assumed.
        """
        client = boto3.client('sts')

        if session_name is None:
            session_name = self.default_session_name

        try:
            session = client.assume_role(RoleArn=self.cross_account_role_arn, RoleSessionName=session_name)
            self.creds = session['Credentials']  # Save for later
            return(session['Credentials'])
        except ClientError as e:
            raise AntiopeAssumeRoleError("Failed to assume role {} in account {} ({}): {}".format(self.cross_account_role_arn,
                self.account_name.encode('ascii', 'ignore'), self.account_id, e.response['Error']['Code']))

    def get_client(self, type, region=None, session_name=None):
        """
        Returns a boto3 client for the service "type" with credentials in the target account.
        Optionally you can specify the region for the client and the session_name for the AssumeRole.
        """
        if 'creds' not in self.__dict__:
            self.creds = self.get_creds(session_name=session_name)
        client = boto3.client(type,
            aws_access_key_id = self.creds['AccessKeyId'],
            aws_secret_access_key = self.creds['SecretAccessKey'],
            aws_session_token = self.creds['SessionToken'],
            region_name = region)
        return(client)

    def get_resource(self, type, region=None, session_name=None):
        """
        Returns a boto3 Resource for the service "type" with credentials in the target account.
        Optionally you can specify the region for the resource and the session_name for the AssumeRole.
        """
        if 'creds' not in self.__dict__:
            self.creds = self.get_creds(session_name=session_name)
        resource = boto3.resource(type,
            aws_access_key_id = self.creds['AccessKeyId'],
            aws_secret_access_key = self.creds['SecretAccessKey'],
            aws_session_token = self.creds['SessionToken'],
            region_name = region)
        return(resource)

    #
    # VPC Methods
    #
    def get_regions(self):
        """Return an array of the regions this account is active in. Ordered with us-east-1 in the front."""
        ec2 = self.get_client('ec2')
        response = ec2.describe_regions()
        output = ['us-east-1']
        for r in response['Regions']:
            if r['RegionName'] == "us-east-1":
                continue
            output.append(r['RegionName'])
        return(output)

    def get_vpc_ids(self):
        """Return a list of VPC ids for the account (as cached in the VPC Table)."""
        # TODO - Add support to filter by region
        output   = []
        vpc_list = []

        vpc_table = self.vpc_table
        response = vpc_table.query(
            IndexName='account-index',
            Select='SPECIFIC_ATTRIBUTES',
            ProjectionExpression='vpc_id',
            Limit=123,
            ConsistentRead=False,
            KeyConditionExpression=Key('account_id').eq(self.account_id)
        )
        while 'LastEvaluatedKey' in response:
            # Means that dynamoDB didn't return the full set, so as for more.
            vpc_list = vpc_list + response['Items']
            response = vpc_table.query(
                IndexName='account-index',
                Select='SPECIFIC_ATTRIBUTES',
                ProjectionExpression='vpc_id',
                Limit=123,
                ConsistentRead=False,
                KeyConditionExpression=Key('account_id').eq(self.account_id),
                ExclusiveStartKey=response['LastEvaluatedKey']
            )
        vpc_list = vpc_list + response['Items']
        # Take the list of vpc_ids and instantiate VPC Objects. Return that list
        for v in vpc_list:
            output.append(v['vpc_id'])
        return(output)

    def get_vpcs(self, region=None):
        """Return a list of VPCs for the account (as cached in the VPC Table). Optionally filter it by region"""
        output   = []
        vpc_list = self.get_vpc_ids()
        for v in vpc_list:
            vpc = VPC(v,  account=self, config=self.config)
            if region is None:
                output.append(vpc)
            else:
                if vpc.region == region:
                    output.append(vpc)
        return(output)

    def get_active_vpcs(self, region=None):
        """Return a list of active VPCs (one or more running instances) for the account. Optionally filter it by region"""
        output   = []
        vpc_list = self.get_vpcs(region)

        # This could also work?
        # nonZeroVpcs =list(filter(lambda x: x.instance_count != '0', vpcs))
        for v in vpc_list:
            if v.is_active():
                output.append(v)
            # FIXME Filter out ones that haven't been updated in last 24 hrs
        return(output)

    #
    # Compliance Functions
    #
    def discover_cft_info_by_resource(self, PhysicalResourceId, region=None, VersionOutputKey='TemplateVersion'):
        """Jump into the account, and ask Cloudformation in that region about the details of a template"""
        output = {}

        try:
            if region is None:
                cfn_client      = self.get_client('cloudformation')
            else:
                cfn_client      = self.get_client('cloudformation', region=region)
        except AntiopeAssumeRoleError:
            logger.error("Unable to assume role looking for {} in {}".format(PhysicalResourceId, self.account_id))
            return(None)

        # Ask Cloudformation "who owns PhysicalResourceId?"
        try:
            stack        = cfn_client.describe_stack_resources(PhysicalResourceId=PhysicalResourceId)
        except ClientError:
            # More error checking needed here.
            # logger.error("Failed to find CFT for {} in {}".format(PhysicalResourceId, self.account_id))
            return(None)  # Nothing else to do. Go home and cry.

        for i in stack['StackResources']:
            if i['PhysicalResourceId'] == PhysicalResourceId:
                output['stack_name'] = i['StackName']
                output['Region'] = region
                break
        else:
            # How is it that describe_stack_resources() returned a stack, but the Resource we searched on wasn't in the resulting dataset?
            logger.error("Found stack {} but resource not present {} in account {}".format(stack_name, PhysicalResourceId, self.account_id))
            return(None)

        # Time to get the stack version
        response     = cfn_client.describe_stacks(StackName=output['stack_name'])
        stack = response['Stacks'][0]
        output['Stack'] = stack
        # Iterate down the outputs till we find the key TemplateVersion. That is our version
        output['template_version'] = False
        if 'Outputs' in stack:
            for o in stack['Outputs']:
                if o['OutputKey'] == VersionOutputKey:
                    output['template_version'] = o['OutputValue']
                    break
            else:
                output['template_version'] = "NotFound"

        # Return the stackname and template_version
        return(output)

    #
    # Database functions
    #
    def update_attribute(self, key, value):
        """
        Update a specific attribute in a specific table for this account.
        key is the column, value is the new value to set
        """
        logger.info(u"Adding key:{} value:{} to account {}".format(key, value, self))
        try:
            response = self.account_table.update_item(
                Key= {
                    'account_id': self.account_id
                },
                UpdateExpression="set #k = :r",
                ExpressionAttributeNames={
                    '#k': key
                },
                ExpressionAttributeValues={
                    ':r': value,
                }
            )
            self.db_record[key] = value
        except ClientError as e:
            raise AccountUpdateError("Failed to update {} to {} in account table: {}".format(key, value, e))

    def get_attribute(self, key):
        """
        Fetches a attribute from the specificed table for the account
        """
        logger.info(u"Getting key:{} from account_table for account {}".format(key, self))
        try:
            response = self.account_table.get_item(
                Key= {
                    'account_id': self.account_id
                },
                AttributesToGet=[key]
            )
            return(response['Item'][key])
        except ClientError as e:
            raise AntiopeAccountLookupError("Failed to get {} from {} in account table: {}".format(key, self, e))
        except KeyError as e:
            raise AntiopeAccountLookupError("Failed to get {} from {} in account table: {}".format(key, self, e))

    def delete_attribute(self, key):
        """
        Delete a attribute from the specificed table for the account
        """
        logger.info(u"Deleting key:{} from account table for account {}".format(key, self))
        table = self.account_table
        try:
            response = table.update_item(
                Key= {
                    'account_id': self.account_id
                },
                UpdateExpression="remove #k",
                ExpressionAttributeNames={
                    '#k': key
                },
                # ExpressionAttributeValues={
                # ':r': value,
                # }
            )
        except ClientError as e:
            raise AntiopeAccountLookupError("Failed to get {} from {} in account table: {}".format(key, self, e))
        except KeyError as e:
            raise AntiopeAccountLookupError("Failed to get {} from {} in account table: {}".format(key, self, e))






# Copyright 2019-2020 Turner Broadcasting Inc. / WarnerMedia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import boto3
from botocore.exceptions import ClientError
from boto3.dynamodb.conditions import Key
import json
import os
import logging

from .vpc import *
from .config import *
from .aws_account import *

import logging
logger = logging.getLogger('antiope.AWSOrganizationMaster')


class AWSOrganizationMaster(AWSAccount):
    """Class to represent an AWS Organization Master Account"""
    def __init__(self, account_id, config=None):
        """ Create a new object representing the AWS orgs master account specified by account_id """
        # Execute any parent class init()
        super(AWSOrganizationMaster, self).__init__(account_id, config=None)
        org_client = self.get_client('organizations')
        describe_response = org_client.describe_organization()
        if describe_response['Organization']['MasterAccountId'] != account_id:
            raise NotAnAWSOrganizationMaster

        self.org_id = describe_response['Organization']['Id']
        self.organization = describe_response['Organization']


    def get_delegated_admin_status(self):
        """
        Call this function to get the list of service principals that have delegated access configured.
        This method will populate a instance attribute called 'org_enabled_service_principals'
        """
        org_client = self.get_client('organizations')

        enabled_response = org_client.list_aws_service_access_for_organization()

        self.org_enabled_service_principals = []
        for sp in enabled_response['EnabledServicePrincipals']:
            self.org_enabled_service_principals.append(sp['ServicePrincipal'])


    def get_delegated_admin_account_for_service(self, service):
        """
        Returns an AWSAccount object for the account that is the delegated admin for the specified service
        Returns None if the service is invalid or not configured in this organization
        """
        try:
            org_client = self.get_client('organizations')

            if not hasattr(self, "org_enabled_service_principals"):
                self.get_delegated_admin_status()

            if "amazonaws.com" not in service:
                service += ".amazonaws.com"

            if service not in self.org_enabled_service_principals:
                logger.error(f"Service {service} is not enabled for this organization.")
                return(None)

            admins_response = org_client.list_delegated_administrators(ServicePrincipal=service)
            if len(admins_response['DelegatedAdministrators']) == 0:
                return(None)
            elif len(admins_response['DelegatedAdministrators']) > 1:
                logger.error(f"More than one DelegatedAdministrators returned for {service}")
                return(None)
            else:
                account = AWSAccount(admins_response['DelegatedAdministrators'][0]['Id'], self.config)
                return(account)
        except ClientError as e:
            if e.response['Error']['Code'] == 'ConstraintViolationException':
                logger.error(f"Error getting delegated admin for {service} from {self.account_id}: {e}")
                return(None)
            else:
                raise

class NotAnAWSOrganizationMaster(Exception):
    '''Exception thrown when an account is not an Org Master account'''



# Copyright 2019-2020 Turner Broadcasting Inc. / WarnerMedia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import boto3
from botocore.exceptions import ClientError
from boto3.dynamodb.conditions import Key
import json
import os
import logging
import datetime
from dateutil import tz
from pprint import pprint

from .aws_account import *
from .config import *

import logging
logger = logging.getLogger('antiope.VPC')

class VPC(object):
    """Class to represent a VPC belonging to an Account"""
    def __init__(self, vpc_id, account=None, config=None):
        '''create a new VPC Instance from the vpc_id from data in the VPC Table'''
        super(VPC, self).__init__()
        self.vpc_id = vpc_id

        if config is None:
            # Assume we're able to get what we need from EnvVars
            self.config = AntiopeConfig()
        else:
            self.config = config

        # # Save these as attributes
        self.account_table = self.config.account_table
        self.vpc_table     = self.config.vpc_table

        # Get the VPC data from the VPC DDB Table. Raise the VPCLookupError if not present
        try:
            response = self.vpc_table.query(
                KeyConditionExpression=Key('vpc_id').eq(self.vpc_id),
                Select='ALL_ATTRIBUTES'
            )
        except ClientError as e:
            raise VPCLookupError("ClientError getting {}: {}".format(self.vpc_id, e))

        try:
            self.db_record = response['Items'][0]

            # Make sure there is a name, if not then use the VPC ID
            if 'name' not in self.db_record:
                self.db_record['name'] = self.vpc_id

            # Convert the response into instance attributes
            self.__dict__.update(self.db_record)
        except IndexError as e:
            raise VPCLookupError("ID {} not found".format(vpc_id))
        except Exception as e:
            logger.error("Got Other error: {}".format(e))

        # We should reference our account in the VPC object. Perhaps the object exists and is part of the init(),
        # perhaps it isn't and we need to create a new one.
        if account is None:
            # This will fail because AWSAccount is not defined and can't be imported due to circular dependencies
            # (AWSAccount requires VPC too)
            self.account = AWSAccount(self.account_id)
        else:
            self.account = account

    def __str__(self):
        """when converted to a string, become the account_id"""
        return(self.vpc_id)

    def __repr__(self):
        '''Create a useful string for this class if referenced'''
        return("<VPC {} >".format(self.vpc_id))

    #
    # Database functions
    #
    def update_vpc_attribute(self, key, value):
        '''    update a specific attribute in a specific table for this account
            table_name should be a valid DynDB table, key is the column, value is the new value to set
        '''
        logger.info(u"Adding key:{} value:{} to VPC {}".format(key, value, self))
        try:
            response = self.vpc_table.update_item(
                Key= {
                    'vpc_id': self.vpc_id
                },
                UpdateExpression="set {} = :r".format(key),
                ExpressionAttributeValues={
                    ':r': value,
                }
            )
            setattr(self, key, value)  # Also update this instance of the object
        except ClientError as e:
            raise AccountUpdateError("Failed to update {} to {} for {}: {}".format(key, value, self, e))

    def get_vpc_attribute(self, key):
        '''
        Pulls a attribute from the specificed table for the account
        '''
        logger.info(u"Getting key: {} for {}".format(key, self))
        try:
            response = self.vpc_table.get_item(
                Key= {
                    'vpc_id': self.vpc_id
                },
                AttributesToGet=[key]
            )
            return(response['Item'][key])
        except ClientError as e:
            raise VPCLookupError("Failed to get {} for {}: {}".format(key, self, e))
        except KeyError as e:
            raise VPCLookupError("Failed to get {} for {}: {}".format(key, self, e))

    #
    # Instance Attributes
    #
    def query_instances(self, instance_state = None):
        '''return an array of dict representing the data from describe_instances()'''
        output = []

        filters = [{'Name': 'vpc-id', 'Values': [self.vpc_id]}]
        if instance_state is not None:
            filters.append({'Name': 'instance-state-name', 'Values': [instance_state]})

        try:
            # Get a boto3 EC2 Resource in this VPC's region.
            ec2 = self.account.get_client('ec2', self.region)
            response = ec2.describe_instances(
                Filters = filters,
                MaxResults = 1000
            )
            while 'NextToken' in response:
                for r in response['Reservations']:
                    for i in r['Instances']:
                        output.append(i)
                response = ec2.describe_instances(
                    Filters = filters,
                    MaxResults = 1000,
                    NextToken = response['NextToken']
                )
            # Done with the while loop (or never entered it) do the last batch
            for r in response['Reservations']:
                for i in r['Instances']:
                    output.append(i)
            return(output)
        except ClientError as e:
            raise VPCLookupError("Failed to query_instances() for {}: {}".format(self, e))

    def query_running_instances(self):
        '''return an array of dict representing the data from describe_instances(). Only includes running instances'''
        return(self.query_instances(instance_state = "running"))

    def discover_instance_count(self):
        '''Get the number of instances in the VPC, update the count in the database'''

        # FIXME (someday) - if the last_updated is within some timeperiod, then don't make the API calls, just used the cached info

        state_count = {
            "pending": 0,
            "running": 0,
            "shutting-down": 0,
            "terminated": 0,
            "stopping": 0,
            "stopped": 0
        }

        instances = self.query_instances()
        for i in instances:
            state = i['State']['Name']
            state_count[state] += 1

        item = {
            "last_updated": str(datetime.datetime.now(tz.gettz('US/Eastern'))),
            "states": state_count
        }

        self.update_vpc_attribute("instance_states", item)
        return(state_count)

    def is_active(self):
        '''Returns true if there are active EC2 Instances running in this VPC'''

        if not hasattr(self, "instance_states"):
            self.discover_instance_count()

        if self.instance_states['running'] > 0:  # FIXME, do something about the instance counts being stale
            return(True)
        return(False)




# Copyright 2019-2020 Turner Broadcasting Inc. / WarnerMedia
# Copyright 2021 Chris Farris <chrisf@primeharbor.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import boto3
from botocore.exceptions import ClientError
from boto3.dynamodb.conditions import Key
import json
import os
import logging
import datetime
from dateutil import tz
from pprint import pprint

from .vpc import *
from .config import *

import logging
logger = logging.getLogger('antiope.ForeignAWSAccount')


class ForeignAWSAccount(object):
    """Class to represent an AWS Account """
    def __init__(self, account_id, config=None):
        """ Create a new object representing the AWS account specified by account_id """
        # Execute any parent class init()
        super(ForeignAWSAccount, self).__init__()

        self.account_id = account_id

        if config is None:
            # Assume we're able to get what we need from EnvVars
            self.config = AntiopeConfig()
        else:
            self.config = config

        # # Save these as attributes
        self.account_table = self.config.account_table

        response = self.account_table.query(
            KeyConditionExpression=Key('account_id').eq(self.account_id),
            Select='ALL_ATTRIBUTES'
        )
        try:
            self.db_record = response['Items'][0]
            # Convert the response into instance attributes
            self.__dict__.update(self.db_record)
            # self.account_name = str(self.account_name.encode('ascii', 'ignore'))
        except IndexError as e:
            raise AntiopeAccountLookupError("ID {} not found".format(account_id))
        except Exception as e:
            logger.error("Got Other error: {}".format(e))

    def __str__(self):
        """when converted to a string, become the account_id"""
        return(self.account_id)

    def __repr__(self):
        """Create a useful string for this class if referenced"""
        return("<ForeignAWSAccount {} >".format(self.account_id))

    #
    # Database functions
    #
    def update_attribute(self, key, value):
        """
        Update a specific attribute in a specific table for this account.
        key is the column, value is the new value to set
        """
        logger.info(u"Adding key:{} value:{} to account {}".format(key, value, self))
        try:
            response = self.account_table.update_item(
                Key= {
                    'account_id': self.account_id
                },
                UpdateExpression="set #k = :r",
                ExpressionAttributeNames={
                    '#k': key
                },
                ExpressionAttributeValues={
                    ':r': value,
                }
            )
        except ClientError as e:
            raise AccountUpdateError("Failed to update {} to {} in account table: {}".format(key, value, e))

    def get_attribute(self, key):
        """
        Fetches a attribute from the specificed table for the account
        """
        logger.info(u"Getting key:{} from account_table for account {}".format(key, self))
        try:
            response = self.account_table.get_item(
                Key= {
                    'account_id': self.account_id
                },
                AttributesToGet=[key]
            )
            return(response['Item'][key])
        except ClientError as e:
            raise AntiopeAccountLookupError("Failed to get {} from {} in account table: {}".format(key, self, e))
        except KeyError as e:
            raise AntiopeAccountLookupError("Failed to get {} from {} in account table: {}".format(key, self, e))

    def delete_attribute(self, key):
        """
        Delete a attribute from the specificed table for the account
        """
        logger.info(u"Deleting key:{} from account table for account {}".format(key, self))
        table = self.account_table
        try:
            response = table.update_item(
                Key= {
                    'account_id': self.account_id
                },
                UpdateExpression="remove #k",
                ExpressionAttributeNames={
                    '#k': key
                },
                # ExpressionAttributeValues={
                # ':r': value,
                # }
            )
        except ClientError as e:
            raise AntiopeAccountLookupError("Failed to get {} from {} in account table: {}".format(key, self, e))
        except KeyError as e:
            raise AntiopeAccountLookupError("Failed to get {} from {} in account table: {}".format(key, self, e))





# Copyright 2019-2020 Turner Broadcasting Inc. / WarnerMedia
# Copyright 2021 Chris Farris <chrisf@primeharbor.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import boto3
from botocore.exceptions import ClientError
from boto3.dynamodb.conditions import Key
import json
import os
import logging

import logging
logger = logging.getLogger('antiope.AntiopeConfig')


class AntiopeConfig(object):
    """Class to represent an AWS Account """
    def __init__(self, config=None, SSMParam=None):
        """ Create a new object representing the AWS account specified by account_id """
        # Execute any parent class init()
        super(AntiopeConfig, self).__init__()

        # The following attributes are expected in the AntiopeConfig Object:
        # * account_table_name
        # * vpc_table_name
        # * role_name
        # * role_session_name
        # * antiope_bucket
        # * dynamodb (boto3 resource)
        # * account_table (dynamodb.Table resource)
        # * vpc_table (dynamodb.Table resource)

        if SSMParam is not None:
            try:
                client = boto3.client('ssm')
                response = client.get_parameter(Name=SSMParam)
                config = json.loads(response['Parameter']['Value'])  # config gets passed to the next conditional.
            except Exception as e:
                logger.critical(f"Failed to get Antiope Config from SSM Parameter Store: {e}")
                exit(1)

        if config is not None:
            try:
                self.account_table_name = config['account_table_name']
                self.vpc_table_name = config['vpc_table_name']
                self.role_name = config['role_name']
                self.role_session_name = config['role_session_name']
            except KeyError as e:
                logger.critical(f"AntiopeConfig passed a config that was missing a key: {e}")
                raise AntiopeDatabaseError(f"AntiopeConfig passed a config that was missing a key: {e}")
        else:
            # Assume things are in the environment
            # This is typical in the Antiope Lambda execution environment
            try:
                self.account_table_name = os.environ['ACCOUNT_TABLE']
                self.vpc_table_name = os.environ['VPC_TABLE']
                if 'ROLE_NAME' not in os.environ:
                    self.role_name = None
                else:
                    self.role_name = os.environ['ROLE_NAME']

                if 'ROLE_SESSION_NAME' not in os.environ:
                    self.role_session_name = "antiope"
                else:
                    self.role_session_name = os.environ['ROLE_SESSION_NAME']
            except KeyError as e:
                logger.critical(f"AntiopeConfig cannot find required key in environment: {e}")
                raise AntiopeDatabaseError(f"AntiopeConfig cannot find required key in environment: {e}")

        # # Save these as attributes
        self.dynamodb      = boto3.resource('dynamodb')
        self.account_table = self.dynamodb.Table(self.account_table_name)
        self.vpc_table     = self.dynamodb.Table(self.vpc_table_name)
        # TODO validate access & existence of these tables


#
# These are the common Exception Classes
#

class AntiopeAssumeRoleError(Exception):
    """raised when the AssumeRole Fails"""
    pass


class AntiopeDatabaseError(Exception):
    """raised when there are issues interacting with the Antiope DynamoDB Tables"""
    pass


class AccountUpdateError(Exception):
    """raised when an update to DynamoDB Fails"""
    pass


class AntiopeAccountLookupError(LookupError):
    """Raised when the Account requested is not in the database"""
    pass


class VPCLookupError(LookupError):
    '''Raised when the VPC requested is not in the database'''
    pass



# Copyright 2019-2020 Turner Broadcasting Inc. / WarnerMedia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from setuptools import setup, find_packages
import os, sys

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
  name='antiope',
  version=os.popen('{} antiope/_version.py'.format(sys.executable)).read().rstrip(),
  author='Chris Farris',
  author_email='chris@room17.com',
  license="Apache License 2.0",
  license_file="LICENSE",
  description='Library for interacting with AWS accounts that are tied to the Antiope framework',
  long_description=long_description,
  long_description_content_type="text/markdown",
  packages=find_packages(),
  py_modules=['antiope'],
  url='https://github.com/WarnerMedia/antiope-aws-module.git',
  python_requires='>=3.6',
  include_package_data=True,
  install_requires=[
    'boto3 >= 1.10.0',
    'botocore >= 1.13.0'
  ]
)
